﻿package de.spielzeugz.test
{
	import flash.display.Sprite;
	import flash.display.Shape;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.text.TextField;
	import flash.geom.ColorTransform;
	import flash.geom.Point;
	import flash.events.*;
	import flash.text.TextFormat;
	
	import net.hires.debug.Stats;
	
	
	[SWF(width='1000',height='560',frameRate='30',backgroundColor='0x000000')]
	
	/**
	 * CopyPixels
	 * 
	 * For further optimisation in Flash use a Vector or linked list instead of 
	 * an Array, a Particle class with typed members instead of generic Objects.
	 * 
	 * @author <a href="mailto:daniel.puhe@spielzeugz.de">Dan</a>
	 */
	public class FlashBlit extends Sprite
	{
		protected var numMovers           :int;
		protected var moverSize           :int;
		protected var movers              :Array;
		protected var canvasBD            :BitmapData;
		protected var canvasBM            :Bitmap;
		protected var point               :Point;
		protected var stageW              :Number;
		protected var stageH              :Number;		
		protected var artwork             :Sprite;
		
		
		
		public function FlashBlit()
		{
			init();
		}
		
		
		public function init() :void
		{
			numMovers = parseInt(loaderInfo.parameters.numObjects) || 150;
			moverSize = parseInt(loaderInfo.parameters.objectSize) || 200;	
			
			stage.align = "TL";
			stage.scaleMode = "noScale";			
			
			// prepare the big bitmap that will render all moving objects
			canvasBM = new Bitmap();
			addChild( canvasBM );
			point = new Point();
			resize( null );
			
			var scale    :Number = moverSize / 200;
			var charX    :Number = scale * 25;
			var charY    :Number = scale * 10;
			var fontSize :Number = scale * 70;			
			
			// prepare the sprite that will be drawn into each object's bitmapData
			var chars :Array = ("!§$%&*+#0123456789¥♠♣♥♪").split("");
			var textField :TextField = new TextField();
			textField.width = textField.height = 200 * scale;
			textField.defaultTextFormat = new TextFormat( "Georgia" , fontSize , 0xFFFFFF );
			textField.x = charX;
			textField.y = charY;
			artwork = new Sprite();
			artwork.addChild( textField );
			
			
			movers = [];
			
			var i :int = numMovers;
			while ( i-- )
			{
				// create a moving object with random velocity
				var m :Object = {};
				m.x  = Math.random() * stageW;
				m.y  = Math.random() * stageH;
				m.vX = 10 - Math.random() * 20;
				m.vY = 10 - Math.random() * 20;
				
				// create amazing artwork here
				textField.text = chars[ i % chars.length ];
				artwork.graphics.clear();
				artwork.graphics.lineStyle();
				artwork.graphics.beginFill( 0xFFFFFF * Math.random() );
				artwork.graphics.drawRect( 0 , 0 , moverSize , moverSize );
				
				// draw artwork to bitmapData
				var bitmapData :BitmapData = new BitmapData( moverSize , moverSize , true , 0 );
				bitmapData.draw( artwork );
				m.bitmapData = bitmapData;
				
				movers[i] = m;
			}
			
			addChild( new Stats() );
			
			stage.addEventListener( Event.RESIZE , resize );
			stage.addEventListener( Event.ENTER_FRAME , run );
		}
		
		
		protected function run( e :Event ) :void
		{
			canvasBD.lock();
			canvasBD.fillRect( canvasBD.rect , 0 );
			
			var i :int = numMovers;
			while ( i-- )
			{
				var m :Object = movers[i];
				
				var nextX :Number = m.x + m.vX;
				var nextY :Number = m.y + m.vY;
				
				// simple stage bounds collision detection
				if( nextX + moverSize > stageW )
				{
					nextX = stageW - moverSize;
					m.vX *= -1;
				}
				else if( nextX < 0 )
				{
					nextX = 0;
					m.vX *= -1;
				}
				
				if( nextY + moverSize > stageH )
				{
					nextY = stageH - moverSize;
					m.vY *= -1;
				}
				else if( nextY < 0 )
				{
					nextY = 0;
					m.vY *= -1;
				}
				
				m.x = nextX;
				m.y = nextY;
				
				
				// copy pixels to canvas bitmapData
				point.x = nextX;
				point.y = nextY;
				canvasBD.copyPixels( m.bitmapData , m.bitmapData.rect , point );
			}
			
			canvasBD.unlock();
		}
		
		
		protected function resize( e :Event ) :void
		{
			stageW = stage.stageWidth;
			stageH = stage.stageHeight;
			if( canvasBD ) canvasBD.dispose();
			canvasBD = new BitmapData( stageW , stageH , true , 0 );
			canvasBM.bitmapData = canvasBD;
			canvasBM.smoothing = true;
		}
		
		
	}
}