﻿package de.spielzeugz.test
{
	import flash.display.*;	
	import flash.events.*;
	import flash.net.URLRequest;
	
	import net.hires.debug.Stats;
	
	
	[SWF(width='1000',height='560',frameRate='30',backgroundColor='0x000000')]
	
	/**
	 * FlashImgsResize
	 * 
	 * Animates MovieClips containing a Bitmap, with resizing.
	 * 
	 * For best code performance you'd rather use a Vector instead of an Array
	 * and a dedicated class with typed members instead of generic Objects.
	 * 
	 * @author <a href="mailto:daniel.puhe@spielzeugz.de">Dan</a>
	 */
	public class FlashImgsResize extends Sprite
	{	
		protected var numMovers           :int;
		protected var moverSize           :int;
		protected var movers              :Array;
		protected var stageW              :Number;
		protected var stageH              :Number;
		protected var loader              :Loader;
		
		
		
		public function FlashImgsResize()
		{
			init();
		}
		
		
		public function init() :void
		{
			numMovers = loaderInfo.parameters.numObjects || 150;
			moverSize = loaderInfo.parameters.objectSize || 200;
			
			loader = new Loader();
			loader.contentLoaderInfo.addEventListener( Event.COMPLETE , setup , false , 0 , true );
			loader.load( new URLRequest( "raspberry.png" ) );
		}		
		
		
		public function setup( e :Event ) :void
		{
			stage.align = "TL";
			stage.scaleMode = "noScale";			
			
			resize( null );
			
			var bitmap :Bitmap = loader.content as Bitmap;
			
			movers = [];
			
			var i :int = numMovers;
			while ( i-- )
			{
				// create a moving object with random velocity
				var m :MovieClip = new MovieClip();
				m.x     = Math.random() * stageW;
				m.y     = Math.random() * stageH;
				m.vX    = 10 - Math.random() * 20;
				m.vY    = 10 - Math.random() * 20;
				m.phase = i;
				
				var bm :Bitmap = new Bitmap( bitmap.bitmapData );
				bm.smoothing = true; // false would be faster
				m.addChild( bm );
				
				movers[i] = m;
				addChild( m );
			}
			
			addChild( new Stats() );
			
			stage.addEventListener( Event.RESIZE , resize );
			stage.addEventListener( Event.ENTER_FRAME , run );
		}
		
		
		protected function run( e :Event ) :void
		{
			var i :int = numMovers;
			while ( i-- )
			{
				var m :MovieClip = movers[i];
				
				var nextX :Number = m.x + m.vX;
				var nextY :Number = m.y + m.vY;
				
				// scale image:
				var size :Number = Math.abs( Math.sin( m.phase ) * moverSize );
				m.width = m.height = size;
				m.phase += 0.05;				
				
				// simple stage bounds collision detection
				if( nextX + size > stageW )
				{
					nextX = stageW - size;
					m.vX *= -1;
				}
				else if( nextX < 0 )
				{
					nextX = 0;
					m.vX *= -1;
				}
				
				if( nextY + size > stageH )
				{
					nextY = stageH - size;
					m.vY *= -1;
				}
				else if( nextY < 0 )
				{
					nextY = 0;
					m.vY *= -1;
				}
				
				m.x = nextX;
				m.y = nextY;
			}
		}
		
		
		protected function resize( e :Event ) :void
		{
			stageW = stage.stageWidth;
			stageH = stage.stageHeight;
		}
		
		
	}
}