﻿package de.spielzeugz.test
{
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.*;
	
	import net.hires.debug.Stats;
	
	
	[SWF(width='1000',height='560',frameRate='30',backgroundColor='0x000000')]
	
	/**
	 * GraphicsLines
	 * 
	 * Draw vector lines continuously each frame.
	 * 
	 * @author <a href="mailto:daniel.puhe@spielzeugz.de">Dan</a>
	 */
	public class FlashLines extends Sprite
	{
		protected var numMovers           :int;
		protected var thickness           :Number = 3;
		
		protected var movers              :Array;
		protected var stageW              :Number;
		protected var stageH              :Number;
		
		
		
		public function FlashLines()
		{
			init();
		}
		
		
		public function init() :void
		{
			numMovers = loaderInfo.parameters.numObjects    || 150;
			thickness = loaderInfo.parameters.lineThickness || 3;
			
			stage.align = "TL";
			stage.scaleMode = "noScale";			
			
			resize( null );
			
			movers = [];
			
			var i :int = numMovers;
			while ( i-- )
			{
				// create a moving object with random velocity
				var m :Object = {};
				m.x  = Math.random() * stageW;
				m.y  = Math.random() * stageH;
				m.vX = 10 - Math.random() * 20;
				m.vY = 10 - Math.random() * 20;
				m.color = 0xFFFFFF * Math.random();
				movers[i] = m;
			}
			
			addChild( new Stats() );
			
			stage.addEventListener( Event.RESIZE , resize );
			stage.addEventListener( Event.ENTER_FRAME , run );
		}
		
		
		protected function run( e :Event ) :void
		{
			graphics.clear();
			graphics.lineStyle( thickness , 0xFFFFFF );
			graphics.moveTo( movers[0].x , movers[0].y );
			
			var i :int = numMovers;
			while ( i-- )
			{
				var m :Object = movers[i];
				
				var nextX :Number = m.x + m.vX;
				var nextY :Number = m.y + m.vY;
				
				
				// simple stage bounds collision detection
				if( nextX > stageW )
				{
					nextX = stageW;
					m.vX *= -1;
				}
				else if( nextX < 0 )
				{
					nextX = 0;
					m.vX *= -1;
				}
				if( nextY > stageH )
				{
					nextY = stageH;
					m.vY *= -1;
				}
				else if( nextY < 0 )
				{
					nextY = 0;
					m.vY *= -1;
				}
				
				m.x = nextX;
				m.y = nextY;
				
				
				// draw the line, toggle between straight and curved lines
				if ( i % 2 == 0 )
				{
					graphics.lineTo( nextX , nextY );
				}
				else
				{
					// use previous mover's location as control point
					var prevMover :Object = movers[ i - 1 ];
					graphics.curveTo( prevMover.x , prevMover.y , nextX , nextY );
				}
			}
		}
		
		
		protected function resize( e :Event ) :void
		{
			stageW = stage.stageWidth;
			stageH = stage.stageHeight;
		}
		
		
	}
}